@@ -1,11 +1,17 @@
+{-# LANGUAGE CPP #-}
+#if MIN_VERSION_base(4,8,1)
+#define HAS_SOURCE_LOCATIONS
+{-# LANGUAGE ImplicitParams #-}
+#endif
+
 -- | Basic definitions for the HUnit library.
 --
 --   This module contains what you need to create assertions and test cases and
---   combine them into test suites. 
+--   combine them into test suites.
 --
---   This module also provides infrastructure for 
---   implementing test controllers (which are used to execute tests). 
---   See "Test.HUnit.Text" for a great example of how to implement a test 
+--   This module also provides infrastructure for
+--   implementing test controllers (which are used to execute tests).
+--   See "Test.HUnit.Text" for a great example of how to implement a test
 --   controller.
 
 module Test.HUnit.Base
@@ -13,10 +19,10 @@ module Test.HUnit.Base
   -- ** Declaring tests
   Test(..),
   (~=?), (~?=), (~:), (~?),
-  
+
   -- ** Making assertions
   assertFailure, {- from Test.HUnit.Lang: -}
-  assertBool, assertEqual, assertString, 
+  assertBool, assertEqual, assertString,
   Assertion, {- from Test.HUnit.Lang: -}
   (@=?), (@?=), (@?),
 
@@ -27,15 +33,23 @@ module Test.HUnit.Base
 
   -- ** Test execution
   -- $testExecutionNote
-  State(..), Counts(..), 
-  Path, Node(..), 
+  State(..), Counts(..),
+  Path, Node(..),
   testCasePaths,
   testCaseCount,
+  Location (..),
   ReportStart, ReportProblem,
   performTest
 )
 where
 
+#ifdef HAS_SOURCE_LOCATIONS
+import           GHC.Stack
+#define with_loc (?loc :: CallStack) =>
+#else
+#define with_loc
+#endif
+
 import Control.Monad (unless, foldM)
 
 
@@ -49,25 +63,28 @@ import Test.HUnit.Lang
 -- -------------------------------
 
 -- | Asserts that the specified condition holds.
-assertBool :: String    -- ^ The message that is displayed if the assertion fails
+assertBool :: with_loc
+              String    -- ^ The message that is displayed if the assertion fails
            -> Bool      -- ^ The condition
            -> Assertion
 assertBool msg b = unless b (assertFailure msg)
 
 -- | Signals an assertion failure if a non-empty message (i.e., a message
 -- other than @\"\"@) is passed.
-assertString :: String    -- ^ The message that is displayed with the assertion failure 
+assertString :: with_loc
+                String    -- ^ The message that is displayed with the assertion failure
              -> Assertion
 assertString s = unless (null s) (assertFailure s)
 
 -- | Asserts that the specified actual value is equal to the expected value.
--- The output message will contain the prefix, the expected value, and the 
+-- The output message will contain the prefix, the expected value, and the
 -- actual value.
---  
+--
 -- If the prefix is the empty string (i.e., @\"\"@), then the prefix is omitted
 -- and only the expected and actual values are output.
-assertEqual :: (Eq a, Show a) => String -- ^ The message prefix 
-                              -> a      -- ^ The expected value 
+assertEqual :: with_loc (Eq a, Show a)
+                              => String -- ^ The message prefix
+                              -> a      -- ^ The expected value
                               -> a      -- ^ The actual value
                               -> Assertion
 assertEqual preface expected actual =
@@ -81,16 +98,16 @@ assertEqual preface expected actual =
 
 -- | Allows the extension of the assertion mechanism.
 --
--- Since an 'Assertion' can be a sequence of @Assertion@s and @IO@ actions, 
+-- Since an 'Assertion' can be a sequence of @Assertion@s and @IO@ actions,
 -- there is a fair amount of flexibility of what can be achieved.  As a rule,
 -- the resulting @Assertion@ should be the body of a 'TestCase' or part of
--- a @TestCase@; it should not be used to assert multiple, independent 
+-- a @TestCase@; it should not be used to assert multiple, independent
 -- conditions.
 --
 -- If more complex arrangements of assertions are needed, 'Test's and
 -- 'Testable' should be used.
 class Assertable t
- where assert :: t -> Assertion
+ where assert :: with_loc t -> Assertion
 
 instance Assertable ()
  where assert = return
@@ -106,7 +123,7 @@ instance (Assertable t) => Assertable (IO t)
 
 -- | A specialized form of 'Assertable' to handle lists.
 class ListAssertable t
- where listAssert :: [t] -> Assertion
+ where listAssert :: with_loc [t] -> Assertion
 
 instance ListAssertable Char
  where listAssert = assertString
@@ -133,13 +150,13 @@ instance ListAssertable Char
 -- 2. Read data from a file, evaluate conditions.
 --
 -- 3. Clean up the file.
--- 
+--
 -- 4. Assert that the side effects of the read operation meet certain conditions.
 --
 -- 5. Assert that the conditions evaluated in step 2 are met.
 type AssertionPredicate = IO Bool
 
--- | Used to signify that a data type can be converted to an assertion 
+-- | Used to signify that a data type can be converted to an assertion
 -- predicate.
 class AssertionPredicable t
  where assertionPredicate :: t -> AssertionPredicate
@@ -158,21 +175,24 @@ infix  1 @?, @=?, @?=
 
 -- | Asserts that the condition obtained from the specified
 --   'AssertionPredicable' holds.
-(@?) :: (AssertionPredicable t) => t          -- ^ A value of which the asserted condition is predicated
+(@?) :: with_loc (AssertionPredicable t)
+                                => t          -- ^ A value of which the asserted condition is predicated
                                 -> String     -- ^ A message that is displayed if the assertion fails
                                 -> Assertion
 predi @? msg = assertionPredicate predi >>= assertBool msg
 
 -- | Asserts that the specified actual value is equal to the expected value
 --   (with the expected value on the left-hand side).
-(@=?) :: (Eq a, Show a) => a -- ^ The expected value
+(@=?) :: with_loc (Eq a, Show a)
+                        => a -- ^ The expected value
                         -> a -- ^ The actual value
                         -> Assertion
 expected @=? actual = assertEqual "" expected actual
 
 -- | Asserts that the specified actual value is equal to the expected value
 --   (with the actual value on the left-hand side).
-(@?=) :: (Eq a, Show a) => a -- ^ The actual value
+(@?=) :: with_loc (Eq a, Show a)
+                        => a -- ^ The actual value
                         -> a -- ^ The expected value
                         -> Assertion
 actual @?= expected = assertEqual "" expected actual
@@ -186,7 +206,7 @@ actual @?= expected = assertEqual "" expected actual
 data Test
     -- | A single, independent test case composed.
     = TestCase Assertion
-    -- | A set of @Test@s sharing the same level in the hierarchy. 
+    -- | A set of @Test@s sharing the same level in the hierarchy.
     | TestList [Test]
     -- | A name or description for a subtree of the @Test@s.
     | TestLabel String Test
@@ -202,7 +222,7 @@ instance Show Test where
 
 -- | Provides a way to convert data into a @Test@ or set of @Test@.
 class Testable t
- where test :: t -> Test
+ where test :: with_loc t -> Test
 
 instance Testable Test
  where test = id
@@ -220,35 +240,38 @@ instance (Testable t) => Testable [t]
 infix  1 ~?, ~=?, ~?=
 infixr 0 ~:
 
--- | Creates a test case resulting from asserting the condition obtained 
+-- | Creates a test case resulting from asserting the condition obtained
 --   from the specified 'AssertionPredicable'.
-(~?) :: (AssertionPredicable t) => t       -- ^ A value of which the asserted condition is predicated
+(~?) :: with_loc (AssertionPredicable t)
+                                => t       -- ^ A value of which the asserted condition is predicated
                                 -> String  -- ^ A message that is displayed on test failure
                                 -> Test
 predi ~? msg = TestCase (predi @? msg)
 
--- | Shorthand for a test case that asserts equality (with the expected 
+-- | Shorthand for a test case that asserts equality (with the expected
 --   value on the left-hand side, and the actual value on the right-hand
 --   side).
-(~=?) :: (Eq a, Show a) => a     -- ^ The expected value 
+(~=?) :: with_loc (Eq a, Show a)
+                        => a     -- ^ The expected value
                         -> a     -- ^ The actual value
                         -> Test
 expected ~=? actual = TestCase (expected @=? actual)
 
--- | Shorthand for a test case that asserts equality (with the actual 
+-- | Shorthand for a test case that asserts equality (with the actual
 --   value on the left-hand side, and the expected value on the right-hand
 --   side).
-(~?=) :: (Eq a, Show a) => a     -- ^ The actual value
-                        -> a     -- ^ The expected value 
+(~?=) :: with_loc (Eq a, Show a)
+                        => a     -- ^ The actual value
+                        -> a     -- ^ The expected value
                         -> Test
 actual ~?= expected = TestCase (actual @?= expected)
 
--- | Creates a test from the specified 'Testable', with the specified 
+-- | Creates a test from the specified 'Testable', with the specified
 --   label attached to it.
 --
 -- Since 'Test' is @Testable@, this can be used as a shorthand way of attaching
--- a 'TestLabel' to one or more tests.  
-(~:) :: (Testable t) => String -> t -> Test
+-- a 'TestLabel' to one or more tests.
+(~:) :: with_loc (Testable t) => String -> t -> Test
 label ~: t = TestLabel label (test t)
 
 
@@ -257,9 +280,9 @@ label ~: t = TestLabel label (test t)
 -- ==============
 
 -- $testExecutionNote
--- Note: the rest of the functionality in this module is intended for 
+-- Note: the rest of the functionality in this module is intended for
 -- implementors of test controllers. If you just want to run your tests cases,
--- simply use a test controller, such as the text-based controller in 
+-- simply use a test controller, such as the text-based controller in
 -- "Test.HUnit.Text".
 
 -- | A data structure that hold the results of tests that have been performed
@@ -278,7 +301,7 @@ type ReportStart us = State -> us -> IO us
 
 -- | Report generator for reporting problems that have occurred during
 --   a test run. Problems may be errors or assertion failures.
-type ReportProblem us = String -> State -> us -> IO us
+type ReportProblem us = Maybe Location -> String -> State -> us -> IO us
 
 -- | Uniquely describes the location of a test within a test hierarchy.
 -- Node order is from test case to root.
@@ -302,13 +325,13 @@ testCaseCount (TestCase _)    = 1
 testCaseCount (TestList ts)   = sum (map testCaseCount ts)
 testCaseCount (TestLabel _ t) = testCaseCount t
 
--- | Performs a test run with the specified report generators.  
+-- | Performs a test run with the specified report generators.
 --
--- This handles the actual running of the tests.  Most developers will want 
--- to use @HUnit.Text.runTestTT@ instead.  A developer could use this function 
--- to execute tests via another IO system, such as a GUI, or to output the 
--- results in a different manner (e.g., upload XML-formatted results to a 
--- webservice).  
+-- This handles the actual running of the tests.  Most developers will want
+-- to use @HUnit.Text.runTestTT@ instead.  A developer could use this function
+-- to execute tests via another IO system, such as a GUI, or to output the
+-- results in a different manner (e.g., upload XML-formatted results to a
+-- webservice).
 --
 -- Note that the counts in a start report do not include the test case
 -- being started, whereas the counts in a problem report do include the
@@ -316,11 +339,11 @@ testCaseCount (TestLabel _ t) = testCaseCount t
 -- only between test case executions.  As a result, the number of test
 -- case successes always equals the difference of test cases tried and
 -- the sum of test case errors and failures.
-performTest :: ReportStart us   -- ^ report generator for the test run start 
+performTest :: ReportStart us   -- ^ report generator for the test run start
             -> ReportProblem us -- ^ report generator for errors during the test run
             -> ReportProblem us -- ^ report generator for assertion failures during the test run
-            -> us 
-            -> Test             -- ^ the test to be executed 
+            -> us
+            -> Test             -- ^ the test to be executed
             -> IO (Counts, us)
 performTest reportStart reportError reportFailure initialUs initialT = do
   (ss', us') <- pt initState initialUs initialT
@@ -334,11 +357,15 @@ performTest reportStart reportError reportFailure initialUs initialT = do
   pt ss us (TestCase a) = do
     us' <- reportStart ss us
     r <- performTestCase a
-    case r of Nothing         -> do return (ss', us')
-              Just (True,  m) -> do usF <- reportFailure m ssF us'
-                                    return (ssF, usF)
-              Just (False, m) -> do usE <- reportError   m ssE us'
-                                    return (ssE, usE)
+    case r of
+      Success -> do
+        return (ss', us')
+      Failure loc m -> do
+        usF <- reportFailure loc m ssF us'
+        return (ssF, usF)
+      Error loc m -> do
+        usE <- reportError loc m ssE us'
+        return (ssE, usE)
    where c@Counts{ tried = n } = counts ss
          ss' = ss{ counts = c{ tried = n + 1 } }
          ssF = ss{ counts = c{ tried = n + 1, failures = failures c + 1 } }
